import os
from multiprocessing import Process


from jax.pyjit import shared_config

from temporalio.client import Client

from jax.cs.storage import StorageObject
from jax.pyjit.dao import StorageKey


from pathlib import Path


class WorkerManager:

    def __init__(self, start_workflow_worker, start_activity_worker) -> None:
        """
        Initialize the Worker with two functions: start_workflow_worker and start_activity_worker.
        Both arguments must be callable.
        """
        if not callable(start_workflow_worker):
            raise TypeError("start_workflow_worker must be a function")
        if not callable(start_activity_worker):
            raise TypeError("start_activity_worker must be a function")
        self.start_workflow_worker = start_workflow_worker
        self.start_activity_worker = start_activity_worker

    def run(self):
        """
        Run the worker processes for workflow and activity workers.
        Checks the sample data bucket and starts two separate processes for workflow and activity workers.
        """
        bname: str = shared_config.TEST_BUCKET_NAME
        if bname:
            self.testSampleDataBucket(bname)

        # One wrinkle in this is that we must to run two processes.
        # Since we signal between the workflow and activities we must
        # not use the same python process for both because of the GIL.
        # This is despite the use of asyncio.
        print("Starting worker on '{}'".format(shared_config.TEMPORAL_URI))
        print("Temporal namespace '{}'".format(shared_config.TEMPORAL_NAMESPACE))

        pw = Process(target=self.start_workflow_worker)
        pa = Process(target=self.start_activity_worker)
        pw.start()
        pa.start()
        pw.join()
        pa.join()

    def testSampleDataBucket(self, bucket: str):
        """
        Test access to the sample data bucket by downloading a file and verifying local storage.
        """
        key: StorageKey = StorageKey(bucket, "/")

        if shared_config.USE_NIO_STORAGE:
            # Check local storage system is working
            uri: str = key.to_uri()
            uri = _hack_uri(uri)

            # We still check path, it may be GCS or S3 or local file
            print("Checking sample data: {}".format(uri))
            wrapped = StorageObject(uri)

            dir: Path = get_dir("test")
            file: Path = Path(dir, Path(uri).name)
            os.makedirs(file.parent, exist_ok=True)
            file.touch()

            with open(file, "wb") as test_file:
                data = wrapped.download_as_bytes()
                test_file.write(data)

            print("Download to: {}".format(file))
            os.remove(file)

        else:
            pass


def _hack_uri(uri: str) -> str:
    """
    Hack the uri to be a local file system.
    This is only for testing purposes.
    """
    # If it's a directory, we need to give a file to StorageObject
    # with jax-cs-storage v<=0.9.3
    try:
        if os.path.isdir(uri):
            for file in os.listdir(uri):
                if file.endswith(".ndpi") and os.path.isfile(os.path.join(uri, file)):
                    uri = os.path.join(uri, file)
                    return uri

    except FileNotFoundError:
        print(f"Directory '{uri}' not found.")

    return None


def get_dir(run_id: str) -> Path:

    cache: Path = Path(shared_config.CACHE_DIR)
    dir: Path = Path(cache, run_id)
    # Make dirs if not existing
    os.makedirs(dir, exist_ok=True)
    return dir


async def connect_client() -> Client:
    """Connect the client."""
    client = await Client.connect(
        shared_config.TEMPORAL_URI, namespace=shared_config.TEMPORAL_NAMESPACE
    )
    return client



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
# Copyright [2025] The Jackson Laboratory

import os


class SharedConfig:
    """Config and Settings Configuration."""

    DEBUG: bool = bool(os.getenv("DEBUG", None))
    TESTING: bool = bool(os.getenv("TESTING", None))

    USE_NIO_STORAGE: bool = bool(os.getenv("USE_NIO_STORAGE", None))
    NIO_STORAGE_ROOT: str = os.getenv("NIO_STORAGE_ROOT", None)

    LOG_LEVEL: str = os.getenv("LOG_LEVEL", "WARNING")
    TEMPORAL_NAMESPACE: str = os.getenv("TEMPORAL_NAMESPACE", "default")
    TEMPORAL_URI: str = os.getenv(
        "TEMPORAL_URI", "localhost:7233"
    )  # Deploy should set to CS cluster location.
    CACHE_DIR: str = os.getenv("CACHE_DIR", "cache")

    TEST_BUCKET_NAME: str = os.getenv("TEST_BUCKET_NAME", None)

    DOWNLOAD_TIMEOUT: int = int(os.getenv("DOWNLOAD_TIMEOUT", "1200"))
    DOWNLOAD_ATTEMPTS: int = int(os.getenv("DOWNLOAD_ATTEMPTS", "4"))

    UPLOAD_TIMEOUT: int = int(os.getenv("UPLOAD_TIMEOUT", "1200"))
    UPLOAD_ATTEMPTS: int = int(os.getenv("UPLOAD_ATTEMPTS", "4"))
    
    CLEAN_TIMEOUT: int = int(os.getenv("CLEAN_TIMEOUT", "300"))
    CLEAN_ATTEMPTS: int = int(os.getenv("CLEAN_ATTEMPTS", "4"))

    # IO operations which come from other workers.
    IO_STORE_QUEUE = os.getenv("STORE_QUEUE", "STORE_TASK_QUEUE")
    IO_DOWNLOAD_OBJECT = os.getenv("IO_DOWNLOAD_OBJECT", "io-temporal-download-object")
    IO_UPLOAD = os.getenv("IO_UPLOAD", "io-temporal-upload")
    IO_CLEAN = os.getenv("IO_CLEAN", "io-temporal-clean")


shared_config = SharedConfig()
